# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import models, fields, api, _, exceptions
from openerp import SUPERUSER_ID
import requests
from lxml import etree as ET
from unidecode import unidecode
import re
import logging

_logger = logging.getLogger(__name__)


class ResPartner(models.Model):
    _inherit = 'res.partner'

    customer_type = fields.Selection(
        [
            ("associated", "Associated"),
            ("non_associated", "Non-associated"),
        ],
        string="Customer type",
        default="associated",
    )
    credipaz_card_additional = fields.Char('Credipaz Card Additional')

    _sql_constraints = [
        ('ref_uniq', 'unique(ref)', 'Reference must be unique!'),
    ]

    @api.multi
    def unlink(self):
        if self.env.user.id != SUPERUSER_ID:
            raise exceptions.ValidationError(_('Only the SUPERUSER can delete partners.'))
        super(ResPartner, self).unlink()

    @api.model
    def do_query_ws(self, reference, raise_on_error=True):
        """
        Returns this info: {'disponible': '0', 'error': '0', 'error_msg': '', 'fecha_consulta': '2017-06-30 18:09:21', 'puede_operar': '1', 'puntos': '0', 'socio': '0'}
        """
        conf_obj = self.env['ir.config_parameter'].sudo()
        ws_url = (conf_obj.get_param('financing.ws.url') or 'http://servicioscpz.credipaz.com/wsMIL/wsMIL.asmx')
        ws_user = conf_obj.sudo().get_param('financing.ws.user')
        ws_pass = conf_obj.sudo().get_param('financing.ws.pass')
        ws_pin = conf_obj.sudo().get_param('financing.ws.pin')
        body = """<?xml version="1.0" encoding="utf-8"?>
        <soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
        <soap:Body>
            <ConsultarDisponible xmlns="https://aplicaciones.credipaz.com:3456/wsMIL/wsMIL.asmx">
            <sUsuario>{ws_user}</sUsuario>
            <sPassword>{ws_pass}</sPassword>
            <sPin>{ws_pin}</sPin>
            <sTipoConsulta>{tipo_consulta}</sTipoConsulta>
            <sValor>{valor}</sValor>
            </ConsultarDisponible>
        </soap:Body>
        </soap:Envelope>
        """.format(tipo_consulta='1', valor=reference, ws_user='odoo', ws_pass='OdooP4sswd', ws_pin='203')  # query by account_code
        # WS Exposes 3 ways: 1=Cuenta, 2=Documento ó 3=Tarjeta
        r = requests.post(ws_url, data=body, headers={'content-type': 'text/xml'})
        res_str = unidecode(r.text)  # Service returns strange characters sometimes :s
        root_text = ET.XML(res_str)[0][0][0].text  # The important string
        _logger.debug(root_text)
        if not root_text:
            raise exceptions.ValidationError(_('Webservice have not returned data.'))
        m = re.match(r".*<FechaConsulta>(?P<fecha_consulta>.*)</FechaConsulta>.*<Cuenta>(?P<cuenta>.*)</Cuenta>.*<Adicional>(?P<adicional>.*)</Adicional>.*<PuedeOperar>(?P<puede_operar>.*)</PuedeOperar>.*<Disponible>(?P<disponible>.*)</Disponible>.*<Puntos>(?P<puntos>.*)</Puntos>.*<SocioRedondo>(?P<socio>.*)</SocioRedondo>.*<Nombre>(?P<nombre>.*)</Nombre>.*<TipoDocumento>(?P<tipo_documento>.*)</TipoDocumento>.*<NroDocumento>(?P<nro_documento>.*)</NroDocumento>.*<Direccion>(?P<direccion>.*)</Direccion>.*<Localidad>(?P<localidad>.*)</Localidad>.*<Provincia>(?P<provincia>.*)</Provincia>.*<PosicionFiscal>(?P<pos_fiscal>.*)</PosicionFiscal>.*<Error>(?P<error>.*)</Error>.*<ErrorMensaje>(?P<error_msg>.*)</ErrorMensaje>", root_text)
        if m:
            res_dict = m.groupdict()
            try:
                assert res_dict['error'] == '0', 'Error in WS: %s [%s]' % (res_dict['error_msg'], res_dict['error'])
            except Exception as e:
                if raise_on_error:
                    raise exceptions.ValidationError('%s' % e)
                else:
                    _logger.warning('CPZ WS, returned a non-zero error code: [%s] %s' % (res_dict['error'], res_dict['error_msg']))
                    return {k: v.strip() for k, v in res_dict.items()}
            else:
                return {k: v.strip() for k, v in res_dict.items()}
        else:
            if raise_on_error:
                raise exceptions.ValidationError(_('WS ERROR: Cannot match the response of the WS'))
            else:
                return {}

    @api.multi
    def get_webservice_data(self):
        self.ensure_one()
        if not self.ref:  # Does not have a credipaz account
            return {
                'partner_id': self.id,
                'available_points': 0,
                'available_credit': 0,
                'ok': False,
                'customer_type': 'non_associated',
            }
        wsres = self.do_query_ws(self.ref)
        _logger.info('Credipaz WS Respose\n%s' % wsres)
        ctype = 'associated' if wsres.get('socio') != '0' else 'non_associated'
        res = {
            'partner_id': self.id,
            'available_points': wsres['puntos'],
            'available_credit': wsres['disponible'],
            'customer_type': ctype,
            'ok': True if wsres['puede_operar'] == '0' else False,
        }
        self.sudo().write({'customer_type': ctype})
        return res

    @api.model
    def create_from_ui(self, partner):
        """ Get data from WS and set it to partner."""
        import pprint; pprint.pprint(partner)
        try:  # Sanitize account position
            partner.update({'property_account_position': int(partner['property_account_position'])})
        except Exception:
            partner.pop('property_account_position', None)
        # No fiscal position->Set it to consumidor final
        if not partner.get('property_account_position', None):
            fp_cons = self.env.ref('l10n_ar_point_of_sale.fiscal_position_final_cons')
            partner['property_account_position'] = fp_cons.id

        if partner.get('id', False):  # Write
            raise exceptions.ValidationError(_('Cannot modify partner from PoS UI.'))
        else:  # Create
            partner_acc_ref = partner.get('ref')
            for k, v in partner.items():
                if v == '':
                    partner.pop(k)
            if (not partner_acc_ref) and (not partner['name']):
                raise exceptions.ValidationError(_('You need to provide a name or account reference for customer creation'))
            if partner_acc_ref:  # Contact ws and use this data instead of what comes by UI for contact creation
                ws_data = self.do_query_ws(partner_acc_ref, raise_on_error=False)
                _logger.info('Credipaz WS Respose\n%s' % ws_data)
                if ws_data['error'] == '0':
                    fp_id = self.env['account.fiscal.position'].search([('name', 'ilike', ws_data.get('pos_fiscal'))]).id or self.env.ref('l10n_ar_point_of_sale.fiscal_position_final_cons').id
                    state_obj = self.env['res.country.state']
                    state_id = state_obj.search([('name', '=ilike', ws_data.get('provincia'))], limit=1).id
                    dt_id = self.env['res.document.type'].search([('name', 'ilike', ws_data.get('tipo_documento'))]).id
                    ctype = 'associated' if ws_data.get('socio') != '0' else 'non_associated'
                    partner = {
                        'street': ws_data.get('direccion'),
                        'city': ws_data.get('localidad'),
                        'name': ws_data.get('nombre'),
                        'vat': ws_data.get('nro_documento'),
                        'property_account_position': fp_id,
                        'state_id': state_id,
                        'document_type_id': dt_id,
                        'customer_type': ctype,
                        'ref': partner_acc_ref,  # ws_data.get('cuenta', '')
                        'country_id': self.env.ref('base.ar').id,
                        'credipaz_card_additional': ws_data.get('adicional', ''),
                    }
                else:  # Error in ws return: No data to retrieve
                    raise exceptions.ValidationError(_('Invalid account reference, maybe is a spelling mistake or the customer is not present in WS'))
            else:
                partner.pop('ref', None)  # Remove ref to prevent empty string as reference
#                if not partner.get('vat'):
#                    raise exceptions.ValidationError(_('VAT Number required'))
                # Custom check
                req_fields = {
                    'street': 'Calle',
                    'city': 'Ciudad',
                    'zip': 'CP',
                    'country_id': 'Pais',
                    'phone': 'Telefono',
                    'email': 'Correo electronico',
                    'vat': 'Documento',
                }
                missing_fields = list()
                for k, v in req_fields.items():
                    if not partner.get(k):
                        missing_fields.append(v)
                if missing_fields:
                    missing_fields_str = ', '.join(missing_fields)
                    raise exceptions.ValidationError(_('%s requerido(s)') % missing_fields_str)

                # End custom check
                cf_fiscal_position = self.env.ref('l10n_ar_point_of_sale.fiscal_position_final_cons')
                cf_document_type = self.env.ref('base_vat_ar.document_dni')
                partner['property_account_position'] = cf_fiscal_position.id
                partner['document_type_id'] = cf_document_type.id
                partner['customer_type'] = 'non_associated'
        res = super(ResPartner, self).create_from_ui(partner)
        return res
