# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import api, fields, models, exceptions, _
# from openerp.osv import fields, osv
from openerp.osv import fields as old_fields
from openerp.osv import osv as old_osv
from openerp.addons import decimal_precision as dp
from openerp.addons.point_of_sale.point_of_sale import pos_order
from datetime import datetime
from openerp.tools import DEFAULT_SERVER_DATETIME_FORMAT
import re
import logging

_logger = logging.getLogger(__name__)


class pos_order_7(old_osv.osv):
    _inherit = 'pos.order'

    _columns = {
        'amount_total': old_fields.function(pos_order._amount_all, string='Total', digits_compute=dp.get_precision('Account'),  multi='all', store=True),
    }


class PaymentCard(models.Model):
    _name = 'payment.card'
    _description = 'Represents any kind of bank card (debit or credit)'
    _sql_constraints = [
        ('name_uq', 'unique(name)', 'Name of card must be unique!'),
    ]

    name = fields.Char('Card Name', required=True)


class POSOrder(models.Model):
    _name = 'pos.order'
    _inherit = ['pos.order', 'mail.thread']

    def get_pids_to_notify_salesman_change(self, cr, uid, obj, ctx=None):
        if len(obj.message_ids.ids) == 1:
            return []  # First salesman change(order created)
        all_follower_ids = obj.message_follower_ids.ids
        cr.execute("select id from res_users where partner_id in %(pid)s", {'pid': tuple(all_follower_ids)})
        all_follower_uids = [i[0] for i in cr.fetchall()]
        uids = obj.user_id.ids
        all_pids = list(set(all_follower_uids + uids))
        return all_pids

    _track = {
        'user_id': {
            'mail.mt_comment': get_pids_to_notify_salesman_change,
        },
    }

    quota_qty = fields.Integer(string='Quota Qty.', default=1)
    quota_amount = fields.Float(string='Quota Amount', digits=dp.get_precision('Financing Amount'))
    points_qty = fields.Float(string='Qty. Points', default=0, compute="compute_qty_points", store=True)
    user_id = fields.Many2one('res.users', 'Salesman', help="Person who uses the the cash register. It can be a reliever, a student or an interim employee.", track_visibility="onchange")
    consignation = fields.Boolean('Consignation')

    # @api.one
    # @api.depends('lines', 'lines.price_unit', 'lines.qty', 'lines.price_subtotal', 'lines.price_subtotal_incl')
    # def _compute_amount(self):
    #     import ipdb; ipdb.set_trace()
    #     amounts = self._amount_all(None, [])
    #     self.amount_total = amounts.get(self.id, {'amount_total': 0})['amount_total']

    @api.multi
    def get_pid_of_managers(self):
        """
        Get the managers of a pos_configuration
        """
        self.ensure_one()
        q_params = {
            'config_id': self.session_id.config_id.id,
        }
        q = """
            WITH q1 AS (
                SELECT res_id FROM ir_model_data WHERE model ~ 'res.group' AND name ~ 'group_pos_manager'
            ) SELECT rp.id FROM res_users ru
                JOIN res_partner rp ON rp.id = ru.partner_id
                JOIN pos_config pc ON pc.id = ru.pos_config
                JOIN res_groups_users_rel rgur ON rgur.uid = ru.id
                JOIN q1 ON q1.res_id=rgur.gid WHERE pc.id=%(config_id)s;"""
        self._cr.execute(q, q_params)
        pids = [i[0] for i in self._cr.fetchall()]
        return pids

    @api.multi
    def message_subscribe(self, partner_ids, subtype_ids=None):
        """
        Add the pos.config managers as followers
        """
        final_pids = list(set(partner_ids + self.get_pid_of_managers()))
        res = super(POSOrder, self).message_subscribe(final_pids, subtype_ids)
        return res

    @api.model
    def create(self, vals):
        original_order_id = vals.get('return_order_id')
        if original_order_id:
            oo_browse = self.env['pos.order'].browse(int(original_order_id))
            vals['quota_amount'] = oo_browse.quota_amount
            vals['quota_qty'] = oo_browse.quota_qty
        res = super(POSOrder, self).create(vals)
        return res

    @api.multi
    def do_complete_return(self):
        for order in self:
            if order.is_return_order:
                raise exceptions.ValidationError(_('This is a Return Order, cannot be refunded again'))
            if order.return_status != '-':
                raise exceptions.ValidationError(_('This order is in a state that cannot be refunded: %s') % order.return_status)
            # Perform Return
            new_order_lines = []
            new_payment_lines = []
            # Search a session that can be used for this refund
            pos_session_id = self.env['pos.session'].search([
                ('config_id', '=', order.session_id.config_id.id),
                ('state', '=', 'opened')
            ])
            if len(pos_session_id) != 1:
                raise exceptions.ValidationError(_('Cannot found a PoS with state opened for the config %s') % order.session_id.config_id.name)
            new_seq_nr = pos_session_id.sequence_number + 1
            for line in order.lines:
                line_vals = {}
                line_vals['qty'] = -line.qty
                line_vals['price_unit'] = line.price_unit
                line_vals['prodlot_id'] = line.prodlot_id.id
                line_vals['product_id'] = line.product_id.id
                line_vals['discount'] = line.discount
                line_vals['guarantee_lot_id'] = line.guarantee_lot_id.id
                new_order_lines.append(line_vals)
            st_obj = self.env['account.bank.statement']
            for payment in order.statement_ids:
                p_vals = {}
                statement_to_use = st_obj.search([
                    ('pos_session_id', '=', pos_session_id.id),
                    ('journal_id', '=', payment.journal_id.id),
                    ('state', '=', 'open')
                ])
                if len(statement_to_use) != 1:
                    raise exceptions.ValidationError(_('Cannot found statement available for PoS Session %s, journal %s and state open') % (pos_session_id.name, payment.journal_id.name))
                p_vals['account_id'] = payment.account_id.id
                p_vals['amount'] = -payment.amount
                p_vals['card_number'] = payment.card_number
                p_vals['card_id'] = payment.card_id.id
                p_vals['journal_id'] = payment.journal_id.id
                p_vals['name'] = fields.Datetime.now()
                p_vals['statement_id'] = statement_to_use.id
                new_payment_lines.append(p_vals)
            order_data = {
                'amount_paid': -order.amount_paid,
                'amount_return': 0,
                'amount_tax': -order.amount_tax,
                'amount_total': -order.amount_total,
                'is_return_order': True,
                'lines': [(0, 0, l) for l in new_order_lines],
                'name': '%s(r)' % order.name,
                'partner_id': order.partner_id.id,
                'pos_session_id': pos_session_id.id,
                'quota_amount': order.quota_amount,
                'quota_qty': order.quota_qty,
                'return_order_id': order.id,
                'return_status': 'Fully-Returned',
                'sequence_number': new_seq_nr,
                'statement_ids': [(0, 0, i) for i in new_payment_lines],
                # 'uid': '%s(r)' % order.uid,
                'user_id': order.user_id.id,
                'pricelist_id': order.pricelist_id.id,
            }
            data_lst = [{
                'data': order_data,
                # 'id': '00001-071-0030',
                'to_invoice': True,
            }]  # Then call create from UI
            self.sudo(order.create_uid).create_from_ui(data_lst)

    @api.depends("statement_ids")
    def compute_qty_points(self):
        for order in self:
            order.points_qty = sum(order.statement_ids.filtered(lambda x: x.journal_id.use_points is True).mapped('qty_points'))

    @api.model
    def get_card_institution(self, card_number):
        if not card_number:
            return '_'
        if card_number.startswith('3'):
            return 'AMEX'
        elif card_number.startswith('4'):
            return 'VISA'
        elif card_number.startswith('5'):
            return 'MASTERCARD'
        else:
            return '_'

    @api.model
    def add_payment(self, order_id, data):
        res = super(POSOrder, self).add_payment(order_id, data)  # res is statement_id
        aj = self.env['account.journal'].browse(data['journal'])
        if aj.pos_payment_type == 'financial':  # write card number  && card_id
            absl = self.env['account.bank.statement.line'].search([
                ('pos_statement_id', '=', order_id),
                '&',
                ('journal_id', '=', data.get('journal')),
                ('amount', '=', data.get('amount')),
            ])
            card_number = data.get('card_number', '_')
            card_id = data.get('card_id')
            absl.write({
                'card_number': card_number,
                'card_id': card_id
            })
        return res

    @api.multi
    def collect_data_for_ws(self):
        self.ensure_one()
        res = {}
        # Now process data from order
        res['cashier_name'] = self.user_id.name
        am_cc = sum(self.statement_ids.filtered(lambda x: x.journal_id.credipaz_card is True).mapped('amount'))
        res['amount_cc'] = am_cc
        am_pt = sum(self.statement_ids.filtered(lambda x: x.journal_id.use_points is True).mapped('amount'))
        payment_codes = self.statement_ids.mapped('journal_id').mapped('code')
        # financial_statements = self.statement_ids.filtered(lambda x: x.journal_id.pos_payment_type == 'financial')
        # am_all_financial_capital = sum(financial_statements.mapped('amount'))
        # if 'TAR' in payment_codes:
        #     am_total_financed = (self.quota_qty * self.quota_amount) + sum(self.statement_ids.filtered(lambda x: x.journal_id.credipaz_card is False and x.journal_id.pos_payment_type == 'financial').mapped('amount'))
        # else:
        # am_total_financed = self.quota_qty * self.quota_amount
        res['amount_pt'] = am_pt
        res['qty_quota'] = self.quota_qty
        res['amount_quota'] = self.quota_amount
        res['credipaz_payment_code'] = self.statement_ids.get_credipaz_payment_code()
        res['payments'] = []
        for payment in self.statement_ids:
            if payment.journal_id.pos_payment_type == 'financial':
                # card_split = payment.card_number.split(': ')
                # card_name = card_split[0]
                # card_number = card_split[1] if len(card_split) == 2 else ''
                card_name = payment.card_id.name or '_'
                card_number = payment.card_number
                account_card = self.partner_id.ref or '' if payment.journal_id.credipaz_card is True else ''
                member = self.partner_id.credipaz_card_additional if payment.journal_id.credipaz_card is True else ''
                term = self.quota_qty
                capital = payment.amount
                if payment.journal_id.credipaz_card:
                    amount = self.quota_qty * self.quota_amount
                else:
                    # amount = (payment.amount / am_all_financial_capital) * am_total_financed
                    if 'TAR' in payment_codes:
                        amount = payment.amount
                    else:
                        amount = self.quota_qty * self.quota_amount
                # Normalize amount decimals
                amount = round(amount, 2)
                if abs(amount - capital) < 0.10: # TODO [ssaid@21/12/2017]: fix this solution
                    amount = capital
                cp_type = 'TAR' if payment.journal_id.credipaz_card is True else 'TEX'
                if payment.journal_id.credipaz_card:
                    cp_type = 'TAR'
                elif payment.journal_id.mp_card:
                    cp_type = 'MPA'
                else:
                    cp_type = 'TEX'
            elif payment.journal_id.pos_payment_type == 'cash':
                if payment.journal_id.use_points:
                    member = self.partner_id.credipaz_card_additional or ''
                    account_card = self.partner_id.ref or ''
                    cp_card = self.statement_ids.filtered('journal_id.credipaz_card')
                    term = 0
                    amount = payment.qty_points
                    capital = payment.qty_points
                    cp_type = 'PUN'
                    if cp_card:
                        #card_split = cp_card[0].card_number.split(': ')
                        #card_name = card_split[0]
                        #card_number = card_split[1]
                        card_name = cp_card.card_id.name or '_'
                        card_number = cp_card.card_number
                    else:
                        card_name = 'CABAL'
                        card_number = ''
                else:  # cash
                    card_name = ''
                    card_number = ''
                    account_card = ''
                    term = 0
                    member = ''
                    amount = payment.amount
                    capital = payment.amount
                    cp_type = 'EFE'
            else:  # Should never enter here!
                print 'never enter here!'
                card_name = ''
                card_number = ''
                account_card = ''
                member = ''
                term = 0
                amount = payment.amount
            payment_data = {
                'num_card': card_number,
                'brand_card': card_name,
                'account_card': account_card,
                'type': cp_type,
                'amount': str(amount),
                'capital': str(capital),
                'term': str(term),
                'member': member,
            }
            res['payments'].append(payment_data)
        res['products'] = []
        for line in self.lines:
            prod_data = {
                'code': line.product_id.default_code or '',
                'desc': line.product_id.name or '',
                'serial_nr': line.prodlot_id.name or '',
                'price_unit': str(line.price_subtotal_incl),
            }
            res['products'].append(prod_data)
        return res

    @api.model
    def create_from_ui(self, orders):
        for order in orders:
            is_refund = order['data'].get('is_return_order')
            if is_refund:
                return_order_id = order['data'].get('return_order_id')  # Order that generates this refund
                self._cr.execute("SELECT id FROM pos_order WHERE return_order_id=%s" % return_order_id)
                other_refund_count = self._cr.rowcount
                if other_refund_count:  # Pop from orders cause is already been refunded
                    _logger.warning('The order %s has already been refunded. Ignore.' % order['data']['name'])
                    orders.remove(order)
        res = super(POSOrder, self).create_from_ui(orders)
        # orders = self.browse(res)
        # for order in orders:
        #     order.collect_data_for_ws()
        return res

    @api.model
    def _order_fields(self, ui_order):
        res = super(POSOrder, self)._order_fields(ui_order)
        res["quota_qty"] = ui_order["quota_qty"]
        res["quota_amount"] = ui_order["quota_amount"]
        res["consignation"] = ui_order.get("consignation", False)
        return res

    @api.model
    def _payment_fields(self, ui_paymentline):
        res = super(POSOrder, self)._payment_fields(ui_paymentline)
        card_id = ui_paymentline.get('card_id', False)
        if not card_id:
            _logger.warning('PoS has not given the appropiate card_id')
        res.update({
            'card_number': ui_paymentline['card_number'],
            'card_id': card_id,
        })
        return res


class PosConfig(models.Model):
    _inherit = 'pos.config'

    display_price_with_taxes = fields.Boolean(
        string='Price With Taxes',
        help="Display Prices with taxes on POS"
    )
    pos_ar_name = fields.Char('Point of Sale Name')
    internal_pos_name = fields.Char('Internal Point of sale name')
    stock_location_transit_id = fields.Many2one('stock.location', string='Stock Location(Transit)', domain=[('usage', '=', 'internal')], required=True)
    pos_ar_consignation = fields.Many2one('pos.ar', 'Pos consignation')

    @api.multi
    def write(self, values):
        res = super(PosConfig, self).write(values)
        if values.has_key('stock_location_id'):  # An ir.rule depends on this and thus cached, if it is not invalidated wrong rules are applied
            self.env['ir.rule'].clear_cache()
        return res



class pos_session(models.Model):
    _inherit = 'pos.session'

    @api.multi
    def view_problematic_orders(self):
        self.ensure_one()
        self._cr.execute("SELECT id FROM pos_order WHERE invoice_id IS NULL AND session_id=%s" % self.id)
        order_not_invoiced_ids = [i[0] for i in self._cr.fetchall()]
        self._cr.execute("SELECT po.id FROM pos_order po JOIN account_invoice ai ON ai.id=po.invoice_id WHERE po.session_id=%s AND ai.state ~ 'draft'" % self.id)
        order_with_invalid_inv_ids = [i[0] for i in self._cr.fetchall()]
        final_ids = list(set(order_not_invoiced_ids + order_with_invalid_inv_ids))
        return {
            'name': _('Orders To Process'),
            'type': 'ir.actions.act_window',
            'view_type': 'form',
            'view_mode': 'tree,form',
            'res_model': 'pos.order',
            'domain': [('id', 'in', final_ids)],
            'target': 'current',
        }


    def wkf_action_close(self, cr, uid, ids, context=None):
        po_obj = self.pool['pos.order']
        aml_obj = self.pool['account.move.line']
        # Call regular workflow
        res = super(pos_session, self).wkf_action_close(
            cr, uid, ids, context=context)

        # Get All Pos Order invoiced during the current Sessions
        po_ids = po_obj.search(cr, uid, [
            ('session_id', 'in', ids),
            ('invoice_id', '!=', False),
        ], context=context)
        po_ids_not_invoiced = po_obj.search(cr, uid, [
            ('session_id', 'in', ids),
            ('invoice_id', '=', False),
        ], context=context)
        if po_ids_not_invoiced:
            raise exceptions.ValidationError(_('Cannot close session due to orders that has not been invoiced. Please invoice them (%s)') % len(po_ids_not_invoiced))
        cr.execute("""
        SELECT count(*) FROM account_invoice ai
        JOIN pos_order po ON po.invoice_id = ai.id
        JOIN pos_session ps ON ps.id = po.session_id
        WHERE ps.id IN (%s) AND ai.state ~ 'draft';
        """, (tuple(ids),))
        count_ai_draft = cr.fetchone()[0]
        if count_ai_draft:
            raise exceptions.ValidationError(_('Cannot close session, you have invoices in draft. Please validate them (%s)') % count_ai_draft)
        for po in po_obj.browse(cr, uid, po_ids, context=context):
            # We're searching only account Invoices that has been payed
            # In Point Of Sale
            #if not po.invoice_id.forbid_payment:
            #    continue

            # Search all move Line to reconcile in Sale Journal
            aml_sale_total = 0
            aml_sale_ids = []
            receivable_or_payable_aml = po.invoice_id.move_id.line_id.filtered(lambda x: x.account_id.type in ('receivable', 'payable') and not x.reconcile_id.id)
            if not receivable_or_payable_aml:
                continue
            aml_sale_ids = receivable_or_payable_aml.ids
            aml_sale_total = sum([i.debit - i.credit for i in receivable_or_payable_aml])

            aml_payment_ids = []
            aml_payment_total = 0
            # Search all move Line to reconcile in Payment Journals
            aml_ids = aml_obj.search(cr, uid, [
                ('pos_order_id', '=', po.id),
                ('reconcile_id', '=', False)], context=context)
            for aml in aml_obj.browse(
                    cr, uid, aml_ids, context=context):
                if (aml.account_id.type == 'receivable'):
                    aml_payment_ids.append(aml.id)
                    aml_payment_total += aml.debit - aml.credit

            # Try to reconcile
            if aml_payment_total != - aml_sale_total:
                # Unable to reconcile
                _logger.warning(
                    "Unable to reconcile the payment of %s #%s."
                    "(partner : %s), %s != %s" % (
                        po.name, po.id, po.partner_id.name, aml_payment_total, aml_sale_total))
            else:
                try:
                    aml_obj.reconcile(
                        cr, uid, aml_payment_ids + aml_sale_ids, 'manual',
                        False, False, False, context=context)
                except Exception as e:
                    _logger.warning('Unable to reconcile pos.order #%s \nAML_SALE_IDS: %s with AML_PAYMENT_IDS: %s\n%s' % (po.id, aml_sale_ids, aml_payment_ids, e))

        return res
