# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

#import itertools
import json

from openerp import _, api, fields, models
import logging

_logger = logging.getLogger(__name__)


class ProductProduct(models.Model):
    _inherit = 'product.product'

    @api.model
    def search_read(self, domain=None, fields=None, offset=0, limit=None, order=None):
        if 'financing_tables' in fields:
            _logger.warning('financing_tables is being read. This fields should not be read anymore.')
            fields.remove('financing_tables')
            simple_res = super(ProductProduct, self).search_read(domain, fields, offset, limit, order)
            pos_config_id = self.env.user.pos_config.id
            _logger.info(_("Getting Financing Tables for %s" % self.env.user.pos_config.name))
            self._cr.execute("""
                SELECT id, '{"%s": {'||SUBSTRING(financing_tables FROM '"%s": {(.*?)}')||'}}' financing FROM product_product WHERE active is true;
            """ % (pos_config_id, pos_config_id))
            financings_list = self._cr.fetchall()
            financing_dict = {i[0]: i[1] for i in financings_list if i[1]}
            for prod in simple_res:
                try:
                    prod['financing_tables'] = financing_dict[prod['id']]
                except Exception:
                    _logger.warning('Cannot setup financing table for product.product #%s: %s' % (prod['id'], prod['default_code']))
            _logger.info(_("Getting Financing Tables for %s [DONE]" % self.env.user.pos_config.name))
            return simple_res
        res = super(ProductProduct, self).search_read(domain, fields, offset, limit, order)
        return res

    @api.depends("financings", "financings.quota_amount")
    def compute_financing_tables(self):
        _logger.warning('financing_tables has been deprecated. Now they are computed in JS. This field is goind to be removed.')
        # table_template = """
        # <table class="tooltip_table">
        #     <tr>
        #         <th class="tooltip_table_header">{}</th>
        #         <th class="tooltip_table_header">{}</th>
        #         <th class="tooltip_table_header">{}</th>
        #         <th class="tooltip_table_header">{}</th>
        #         <th class="tooltip_table_header">{}</th>
        #     </tr>
        #     %s
        # </table>
        # """.format(_("Quota"), _("Quota Amount"), _("Total"), _("A.P.R."), _("T.F.C."))
        #
        # data_template = """
        # <tr>
        #     <td class="tooltip_table_data">{f.financing_config_id.quota_qty}</td>
        #     <td class="tooltip_table_data">{f.quota_amount_taxed:.2f}</td>
        #     <td class="tooltip_table_data">{f.total_taxed:.2f}</td>
        #     <td class="tooltip_table_data">{f.apr:.3f}</td>
        #     <td class="tooltip_table_data">{f.tfc:.3f}</td>
        # </tr>
        # """
        # no_financing_msg = _('This product is not properly configured!')
        # empty_json = json.dumps(no_financing_msg)
        # for product in self:
        #     if not product.financings:
        #         product.financing_tables = empty_json
        #         continue
        #
        #     # Build table rows
        #     rows = {}
        #     for financing in product.financings:
        #         rows.setdefault(financing.pos_config_id.id, {}).setdefault(
        #             financing.customer_type, []).append(data_template.format(f=financing))
        #
        #     # Join rows with the table header
        #     tables = {}
        #     for pos_config_id, data in rows.items():
        #         for customer_type, row in data.items():
        #             table = table_template % "\n".join(row)
        #             tables.setdefault(pos_config_id, {})[customer_type] = table
        #
        #     #product.financing_tables = ";".join(itertools.chain(*tables.items()))
        #     product.financing_tables = json.dumps(tables)

    @api.onchange("lst_price")
    def onchange_lst_price(self):
        ret = {}
        if self.sale_ok and self.financings:
            title = _("Financing should be updated")
            msg = _("Remember to press 'Update financings' button to update the Quota Amount!")
            ret["warning"] = {"title": title, "message": msg}

        return ret

    @api.multi
    def button_update_financings(self):
        for product in self:
            financings = []
            _logger.info('Update financing for product %s' % product.id)
            for financing in product.financings:
                config = financing.financing_config_id
                quota_amount = 0
                if config:
                    lst_price = product.lst_price
                    quota_qty = config.quota_qty
                    total_rate = config.total_rate or 0
                    quota_amount = (lst_price * (1 + total_rate)) / quota_qty

                financings.append((1, financing.id, {"quota_amount": quota_amount}))

            product.write({"financings": financings})
        return True

    financings = fields.One2many('product.financing', 'product_id', string='Financings')
    financing_tables = fields.Text(string='Financing Tables', compute="compute_financing_tables", store=True)
    no_interest = fields.Boolean('Disable interest', help="If enabled the interest wont be calculated")

    @api.multi
    def write(self, values):
        """
        Overriden to trigger update in financing tables
        """
        self.ensure_one()
        res = super(ProductProduct, self).write(values)
        if 'lst_price' in values:
            self.button_update_financings()
        return res
