# -*- coding: utf-8 -*-
from openerp import models, fields, api, exceptions, _
from lxml import etree


class stock_picking(models.Model):
    _inherit = 'stock.picking'

    @api.cr_uid_ids_context
    def do_transfer(self, cr, uid, picking_ids, context=None):
        res = super(stock_picking, self).do_transfer(cr, uid, picking_ids, context)
        # Recompute locations of lots involved
        for picking in self.browse(cr, uid, picking_ids, context=context):
            lot_ids = picking.pack_operation_ids.mapped('lot_id')
            lot_ids.action_update_last_location()
        return res


class stock_quant(models.Model):
    _inherit = 'stock.quant'

    @api.model
    def fields_view_get(self, view_id=None, view_type='form', toolbar=False, submenu=False):
        res = super(stock_quant, self).fields_view_get(view_id=view_id, view_type=view_type, toolbar=toolbar, submenu=submenu)
        # spt_id = self._context.get('default_picking_type_id')
        if view_type in ('search', 'tree') and self.env.user.pos_config:
            # spt = self.env['stock.picking.type'].browse(spt_id)
            poscfg = self.env.user.pos_config
            doc = etree.XML(res['arch'])
            locations_pos = [poscfg.stock_location_transit_id.id, poscfg.stock_location_id.id]
            for node in doc.xpath("//filter[@name='my_pos_location']"):
                domain = "[('location_id.id', 'in', %s)]" % locations_pos
                node.set('domain', domain)
            res['arch'] = etree.tostring(doc)
        return res


class stock_production_lot(models.Model):
    _inherit = 'stock.production.lot'

    consignation = fields.Boolean('Consignation')

    @api.multi
    def read(self, fields=None, load='_classic_read'):
        res = super(stock_production_lot, self).read(fields=fields, load=load)
        for record in self:
            if record.consignation:
                if res[0].get('name', False):
                    res[0]['name'] = res[0]['name'] + _(" (Consignation)")
        return res

    @api.multi
    def name_get(self):
        res = super(stock_production_lot, self).name_get()
        for i, record in enumerate(res):
            production_lot = self.browse(record[0])
            if production_lot.consignation:
                record = (record[0], record[1] + _(" (Consignation)"))
                res[i] = record
        return res

    @api.multi
    def action_update_last_location(self):
        for record in self:
            if record.quant_ids.ids:
                last_quant_id = max(record.quant_ids.ids)
                last_quant_data = self.env['stock.quant'].browse(last_quant_id)
                record.last_location_id = last_quant_data.location_id.id
            else:
                record.last_location_id = False
