/******************************************************************************
 *    Financing - Financing on Odoo
 *    Copyright (C) 2017 Eynes
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

function financing_db(instance, module) {

    var PosDBParent = module.PosDB;
    var round_di = instance.web.round_decimals;
    var round_pr = instance.web.round_precision;
    module.PosDB = module.PosDB.extend({

        init: function (options) {
            options = options || {};
            this._super(options);
            this.financing_by_product_id = {};
            this.table_by_product_id = {};
            this.financing_configs = {};
            this.remove_all_orders();
            // ^-- Credipaz orders are to invoice only, so when starting up pos clear all in db
        },
        
        add_partners: function(partners){  // prevent pos from viewing false as string in partner creation
            var self = this;
            for (var i = 0, len = partners.length; i < len; i++) {
                p = partners[i];
                for (var key in p){
                    if (p.hasOwnProperty(key)) {
                        if (p[key] === false) {
                            p[key] = "";
                        }
                    }
                }
            }
            res = this._super(partners);
            return res
        },
        add_products_with_pos: function (products, pos_model) {
            this.add_products(products);
            if (!products instanceof Array) {
                products = [products];
            }
            tax_obj = {};
            for (var i = 0, len = pos_model.taxes.length; i < len; i++) {
                tax_obj[pos_model.taxes[i].id] = pos_model.taxes[i];
            }
            for(var i = 0, len = products.length; i < len; i++){
                var product = products[i];
                var product_id = product.id;
                this.table_by_product_id[product_id] = { false: "" };
                var tables = product.financing_tables;

                try {
                    var tax_to_apply = tax_obj[product.taxes_id[0]];
                    product.price_with_taxes = product.list_price * (1 + tax_to_apply.amount)
                } catch (e) {
                    /* handle error */
                    console.warn(product);
                }


                if (!tables) { continue };

                //var splits = tables.split(";");
                //if (tables.indexOf(";") < 0 || splits.length <= 1) { continue };
                //var tables = JSON.parse(product.financing_tables[pos_model.config.id]);
                var tables_obj = JSON.parse(tables);
                var config_id = pos_model.config.id.toString();
                if ( !tables_obj.hasOwnProperty(config_id) ) { continue };
                this.table_by_product_id[product_id] = tables_obj[config_id];

                //for (var j = 0; j < splits.length; j++) {
                //    //var customer_type = splits[j].trim(); 
                //    //var table = splits[j + 1].trim(); 
                //    j++;
                //}
            }
        },
        get_table: function(product_id, customer_type){
            var self = this;
            var table_header = '<table class="tooltip_table">'
              + '<tr><th class="tooltip_table_header">Cuotas</th>' 
              + '<th class="tooltip_table_header">$</th>' 
              + '<th class="tooltip_table_header">Total</th>' 
              + '<th class="tooltip_table_header">T.E.A.</th>' 
              + '<th class="tooltip_table_header">C.F.T.</th></tr>';
            var closing_tag = '</table>';
            // Table
            try {
                var financings = this.financing_by_product_id[product_id][customer_type];
            } catch (e) {
                return '';
            }
            var table = '';
            try {
                var tax_id = this.product_by_id[product_id].taxes_id[0];
                var tax = window.posmodel.taxes_by_id[tax_id];
                var tax_amount = 1 + tax.amount;
            } catch (e) {
                var tax_amount = 1;  // No tax applied
            }
            if (financings === undefined) {
                return '';
            }
            for (var k = 0; k < financings.length; k++) {
                var financing = financings[k];
                total_round = (round_di(parseFloat(financing.total) || 0, 2) * tax_amount).toFixed(2);
                qa_round = (round_di(parseFloat(financing.quota_amount) || 0, 2) * tax_amount).toFixed(2);
                table += '<tr><td class="tooltip_table_data">' 
                    + financing.quota_qty.toString() + '</td><td class="tooltip_table_data">'
                    + qa_round.toString() + '</td><td class="tooltip_table_data">'
                    + total_round.toString() + '</td><td class="tooltip_table_data">'
                    + financing.apr.toString() + '</td><td class="tooltip_table_data">'
                    + financing.tfc.toString() + '</td></tr>';
            };
            // Table
            var the_table = table_header + table + closing_tag;
            return the_table;
        },
        set_financing_tables: function(financings) {
            // In: Financings
            // Out: Set db.table_by_product_id = {product_id: {customer_type: html_table}}
            var self = this;   
            var res = {};
            var product_ids = Object.keys(this.product_by_id);
            for (var i = 0, len = product_ids.length; i < len; i++) {
                var product_id = parseInt(product_ids[i]); 
                var associated_tab = this.get_table(product_id, 'associated');
                var non_associated_tab = this.get_table(product_id, 'non_associated');
                res[product_id] = {
                    'associated': associated_tab,
                    'non_associated': non_associated_tab
                };
            }
            this.table_by_product_id = res;
        },

        add_financings: function (financings) {
            // build an useful financing map table as follows: product_id,customer_type -> [financings]
            if (!(financings instanceof Array)) {
                financings = [financings];
            }
            //console.log(this);

            for (var i=0; i < financings.length; i++) {
                financing = financings[i];
                // get product_id form the financing
                var product_id = financing.product_id[0];
                if (!this.financing_by_product_id.hasOwnProperty(product_id)) { 
                    // set product_id default map
                    this.financing_by_product_id[product_id] = {};
                }

                var fbp = this.financing_by_product_id[product_id];
                if (!fbp.hasOwnProperty(financing.customer_type)) { 
                    // set customer_type default array
                    this.financing_by_product_id[product_id][financing.customer_type] = [];
                }

                // append current financing to the array
                this.financing_by_product_id[product_id][financing.customer_type].push(financing);
            }
        },

        add_financing_configs: function (financing_configs, pos_model) {
            if (!(financing_configs instanceof Array)) {
                financing_configs = [financing_configs];
            }

            for (var i=0; i < financing_configs.length; i++) {
                config = financing_configs[i];
                if (!this.financing_configs.hasOwnProperty(config.customer_type)) { 
                    // set customer_type default array
                    this.financing_configs[config.customer_type] = [];
                }

                this.financing_configs[config.customer_type][config.quota_qty] = config;
            }
            pos_model.set_enabled_quotas(financing_configs);
        },

        search_financing_by_quota_qty: function (product_id, customer_type, quota_qty) {
            var financings = this.financing_by_product_id[product_id][customer_type]
            for (var i = 0; i < financings.length; i++) {
                var financing = financings[i];
                if (financing.quota_qty === quota_qty) {
                    return financing;
                }
            }
            return null;
        }

    })
}
