/******************************************************************************
*    Point Of Sale - Pricelist for POS Odoo
*    Copyright (C) 2014 Taktik (http://www.taktik.be)
*    @author Adil Houmadi <ah@taktik.be>
*
*    This program is free software: you can redistribute it and/or modify
*    it under the terms of the GNU Affero General Public License as
*    published by the Free Software Foundation, either version 3 of the
*    License, or (at your option) any later version.
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU Affero General Public License for more details.
*    You should have received a copy of the GNU Affero General Public License
*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
******************************************************************************/
function financing_screens(instance, module) {

    var _t = instance.web._t;
    var round_pr = instance.web.round_precision;
    var round_di = instance.web.round_decimals;

    module.ClientListScreenWidget = module.ClientListScreenWidget.extend({
        save_client_details: function(partner) {
            var self = this;
            
            var fields = {}
            this.$('.client-details-contents .detail').each(function(idx,el){
                fields[el.name] = el.value;
            });

            if (!fields.name && !fields.ref) {
                this.pos_widget.screen_selector.show_popup('error',{
                    message: _t('A Customer Name or Customer Account Is Required'),
                });
                return;
            }
            
            if (this.uploaded_picture) {
                fields.image = this.uploaded_picture;
            }

            fields.id           = partner.id || false;
            fields.country_id   = fields.country_id || false;
            fields.ean13        = fields.ean13 ? this.pos.barcode_reader.sanitize_ean(fields.ean13) : false; 

            new instance.web.Model('res.partner').call('create_from_ui',[fields]).then(function(partner_id){
                self.saved_client_details(partner_id);
            },function(err,event){
                //console.info(err);
                event.preventDefault();
                if (err.data.message) {
                    comment = _t(err.data.message);
                } else {
                    comment = _t('Your Internet connection is probably down.');
                }
                self.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Could not Save Changes'),
                    'comment': comment,
                });
            });
        },
    });

    module.PaymentScreenWidget = module.PaymentScreenWidget.extend({
        init: function(parent, options) {
            var self = this;
            this._super(parent,options);
            this.line_card_handler = function(event){
                var node = this;
                while(node && !node.classList.contains('paymentline')){
                    node = node.parentNode;
                }
                if (node) {
                    try{
                        card_number = this.value;
                    }
                    catch(e){
                        card_number = "";
                    }
                    node.line.set_card_number(card_number);
                }
            };
            this.line_mp_input_handler = function(event){
                var node = this;
                while(node && !node.classList.contains('paymentline')){
                    node = node.parentNode;
                }
                if (node) {
                    try{
                        card_id = this.value;
                    }
                    catch(e){
                        card_id = "";
                    }
                    node.line.set_card_id(card_id);
                }
            };
            this.line_change_handler = function(event){
                var node = this;
                while(node && !node.classList.contains('paymentline')){
                    node = node.parentNode;
                }
                if(node){
                    var amount;
                    try{
                        amount = instance.web.parse_value(this.value, {type: "float"});
                    }
                    catch(e){
                        amount = 0;
                    }
                    if (!node.line.cashregister.journal.use_points) {
                        node.line.set_amount(amount);
                    } else {  // Check if we are receiving points or money
                        var repaint = event.which === 13 && true || false;
                        if (repaint) {
                            event.stopPropagation();
                        }
                        if (event.srcElement.id == 'money_input') {
                            node.line.set_amount_and_points(amount, repaint);
                        } else {
                            node.line.set_points_and_amount(amount, repaint);
                        }
                    }
                    //if ( node.line && node.line.cashregister && node.line.cashregister.journal.pos_payment_type == 'cash' ) {
                    //    var currentOrder = self.pos.get('selectedOrder');
                    //    self.change_quota_qty(currentOrder.quota_qty);
                    //}
                }
            };
            this.hotkey_handler = function(event){
                if(event.which === 13){
                    self.validate_order({invoice: true});
                }else if(event.which === 27){
                    self.back();
                }
            };
        },

        validate_order: function(options) {
            var self = this;
            options = options || {};
            var currentOrder = this.pos.get('selectedOrder');
            var plines = currentOrder.get('paymentLines').models;
            var orderlines = currentOrder.get('orderLines').models;
	    var consignation = orderlines[0].consignation
	    for (var i = 0, len = orderlines.length; i < len; i++) {
	        if (orderlines[i].consignation != consignation) {
                    this.pos.pos_widget.screen_selector.show_popup('error',{
                        'message':_t('Error: Consignacion'),
                        'comment': _t('No se puede combinar productos sin consignacion, con productos que si poseen.'),
                    });
		    return;
		}	
	    }
            // Check if it is generic customer raise exception
//            var generic_customer = self.pos.config.default_partner_id[0];
//            var current_customer = currentOrder.get_client().id;
            if (currentOrder.get_client().name === 'Consumidor Final') {
                this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Identificar al cliente'),
                    'comment': _t('No se puede vender al cliente consumidor final'),
                });
                return;
            }
            // Check done
            for (var i = 0; i < plines.length; i++) {
                if (plines[i].cashregister.journal.pos_payment_type == 'financial') {
                    if (plines[i].card_number == null || plines[i].card_number === "") {
                        this.pos.pos_widget.screen_selector.show_popup('error',{
                            'message':_t('Error: Card Number Required'),
                            'comment': _t('You need to provide the customer card number.'),
                        });
                        return;
                    }
                    if (currentOrder.get_quotas_total() == 0) {
                        this.pos.pos_widget.screen_selector.show_popup('error',{
                            'message':_t('Error'),
                            'comment': _t('Invalid quota_amount.'),
                        });
                        return;
                    };
                    if (plines[i].card_id == null || plines[i].card_id === "") {
                        this.pos.pos_widget.screen_selector.show_popup('error',{
                            'message':_t('Error: Card Required'),
                            'comment': _t('You need to provide the card issuer.'),
                        });
                        return;
                    }
                }
            }
            var tot_points_to_use = 0;
            var tot_credit_to_use = 0;
            var points_to_cash = 0;
            var client_points = parseFloat(currentOrder.attributes.client.ws_points || 0);
            var client_credit = parseFloat(currentOrder.attributes.client.ws_credit || 0);
            for (var i = 0; i < plines.length; i++) {
                if (plines[i].cashregister.journal.credipaz_card) {
                    tot_credit_to_use += plines[i].amount;
                }
                if (plines[i].cashregister.journal.use_points) {
                    tot_points_to_use += plines[i].points;
                    points_to_cash = Math.round(((plines[i].points / plines[i].cashregister.journal.points_rate)) * 100) / 100;
                    if (points_to_cash != plines[i].amount) {
                        this.pos.pos_widget.screen_selector.show_popup('error',{
                            'message':_t('Error: Points and Money does not match'),
                            'comment': _t('Maybe you have forgotten to recalculate the points or the amount.'),
                        });
                        return;
                    }
                }
            }
            if ((tot_points_to_use !== 0) && (tot_points_to_use > client_points)) {
                this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Not enough points'),
                    'comment': _t('Trying to use more points than customer has available.'),
                });
                return;
            }
            if (parseFloat(tot_credit_to_use.toFixed(2)) > client_credit) {
                this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t('Error: Not enough credit'),
                    'comment': _t('Trying to use more credit than customer has available in credipaz card.'),
                });
                return;
            }
            if ( ((_.unique(orderlines, false, function(i){return i.product.default_code.substring(0, i.product.default_code.length-1)})).length > 1) && (plines.length > 1) ) {
                this.pos.pos_widget.screen_selector.show_popup('error',{
                    'message':_t("Error: Can't mix Payment Modes"),
                    'comment': _t("If you have more than one product you must use only one Payment Mode."),
                });
                return;
            }
            for (var i=0; i < orderlines.length; i++) {
                var line = orderlines[i];
                if ( line.price == 0 ) {
                    this.pos.pos_widget.screen_selector.show_popup('error',{
                        'message':_t("Error: Can't sell a product at $0"),
                        'comment': line.product.display_name,
                    });
                    return;
                }
            }
	    self.pos.get_order().consignation = consignation
            return this._super(options);
        },

        render_paymentline: function(line){
            var self = this;
            var el_node = this._super(line);
            if (line.cashregister.journal.use_points) {
                var pts_in = el_node.querySelector('input#points_input');
                pts_in.addEventListener('keyup', this.line_change_handler);
                pts_in.addEventListener('click', this.line_click_handler);
            }
            if (line.cashregister.journal.pos_payment_type == 'financial') {
                var card_number_in = el_node.querySelector('input#card_number_input');
                card_number_in.addEventListener('keyup', this.line_card_handler);
                card_number_in.addEventListener('click', this.line_card_handler);
                var mp_input_sel = el_node.querySelector('select#mp_input_sel');
                mp_input_sel.addEventListener('keyup', this.line_mp_input_handler);
                mp_input_sel.addEventListener('click', this.line_mp_input_handler);
            }
            return el_node;
        },

        get_cash_amount: function () {
            var currentOrder = this.pos.get('selectedOrder');
            var paymentlines = currentOrder.get('paymentLines').models;

            var cash_payments = paymentlines.filter(function(p) {
                return p.cashregister.journal.pos_payment_type === 'cash';
            })

            var cash_amount = cash_payments.reduce(function(sum, payment) {
                return sum + payment.amount;
            }, 0)
             
            var any_cp_financial_payments = paymentlines.filter(function(p) {return p.cashregister.journal.credipaz_card == true}).length
            if (any_cp_financial_payments > 0) {
                var not_credipaz_financial_payments = paymentlines.filter(function(p) {
                    return p.cashregister.journal.pos_payment_type === 'financial' && p.cashregister.journal.credipaz_card === false;
                })
                var not_credipaz_financial_payments_amount = not_credipaz_financial_payments.reduce(function(vant, vact, iact, arr) { return vant + vact['amount'] }, 0)
                cash_amount += not_credipaz_financial_payments_amount;
            }

            var cash_amount_fixed = Math.round(cash_amount * 100) / 100;
            return cash_amount_fixed;
        },

        calculate_unit_price: function(line, quota_qty, customer_type, mind_the_payments) {
            var db = this.pos.db;
            var financing = undefined;
            var prod_financing = db.financing_by_product_id[line.product.id];
            if (prod_financing == null) {
                product_financings = [];
            } else {
                var product_financings = prod_financing[customer_type];
                if (product_financings == null) {
                    product_financings = [];
                }
            }
            for (var j=0; j < product_financings.length; j++) {
                if (product_financings[j].quota_qty == quota_qty) {
                    financing = product_financings[j];
                    break;
                }
            }

            var unit_price = 0.0;
            var quota_amount = 0.0;
            var cash_amount = 0.0;

            if ( financing === undefined ) {
                if (quota_qty == 1) {
                    if (line.product.no_interest == true) {
                        unit_price = line.price;
                        quota_amount = unit_price;
                        cash_amount = this.get_cash_amount();
                    } else {
                        unit_price = line.product.list_price;
                        quota_amount = unit_price;
                        cash_amount = this.get_cash_amount();
                    }
                } else {  // quota_qty != 1
                     if (line.product.no_interest == true) {
                        unit_price = line.price;
                        quota_amount = unit_price;
                        cash_amount = this.get_cash_amount();
                    }              
                }
            }
            else if ( mind_the_payments ) {
                var prices = line.get_all_prices();
                cash_amount = this.get_cash_amount();
                if ( cash_amount == 0 || cash_amount == prices.priceWithTax ) {
                    if (line.product.no_interest) {
                        unit_price = line.price;
                        quota_amount = unit_price;
                    } else {
                        dto_to_apply = financing.total * (line.discount / 100);
                        unit_price = financing.total - dto_to_apply;
                        quota_amount = financing.quota_amount;
                    }
                }
                else {
                    // in this case quota_amount is computed later 
                    unit_price = line.recalculate_unit_price(cash_amount, financing);
                }
            }
            else {
                dto_to_apply = financing.total * (line.discount / 100);
                unit_price = financing.total - dto_to_apply;
                quota_amount = financing.quota_amount;
            }

            return {"unit_price": unit_price, "quota_amount": quota_amount, "cash_amount": cash_amount};
        },

        change_quota_qty_guarantee: function(quota_qty){
            /* Handles the quota management for guarantees.
             * We read the coef from financing.config, and apply that to do the calculations
             * */
            var db = this.pos.db;
            var currentOrder = this.pos.get('selectedOrder');
            var customer = currentOrder.get_client();
            var orderlines = currentOrder.get('orderLines').models;
            var paymentlines = currentOrder.get('paymentLines').models;
            var customer_type = customer.customer_type;
            var rate = 1;
            try {
                financing_config = db.financing_configs[customer_type][quota_qty];
                rate += financing_config.total_rate;
            } catch (e) {
                /* handle error */
                console.warn("No financing available for this guarantee");
            }
            // In rate we have 1 or 1.1815
            var totalWithTax = currentOrder.getTotalTaxIncluded();
            var cash_amount = this.get_cash_amount();
            var line = orderlines[0];  // Use the first line to store calculated financing
            currentOrder.quota_qty = quota_qty;
            var remain_to_be_paid = totalWithTax - cash_amount;
            if (remain_to_be_paid > 0) {
                var quota_amount = (remain_to_be_paid * rate) / quota_qty;
                line.quota_amount = quota_amount;
            } else {
                line.quota_amount = 0;
            }
            
            // Updating stuff
            var financing_payments = paymentlines.filter(function(p) {
                return p.cashregister.journal.pos_payment_type === 'financial' && p.cashregister.journal.credipaz_card === true;
            });
            if ( financing_payments.length >= 1 ) {
                var last_paymentline = financing_payments[financing_payments.length - 1];
                var due_left = currentOrder.getDueLeft();
                last_paymentline.set_amount(due_left + last_paymentline.amount);
                this.rerender_paymentline(last_paymentline);
            }
            this.update_financing_summary();
        },

        change_quota_qty: function(quota_qty) {
            var db = this.pos.db;
            var currentOrder = this.pos.get('selectedOrder');
            var customer = currentOrder.get_client();
            var orderlines = currentOrder.get('orderLines').models;
            var paymentlines = currentOrder.get('paymentLines').models;
            var customer_type = customer.customer_type;
            // update quotas quantity on current order
            currentOrder.quota_qty = quota_qty;
            if (currentOrder.is_guarantee_order) {
                return this.change_quota_qty_guarantee(quota_qty);   
            }
            if (orderlines.length == 1) {
                var line = orderlines[0];
                var dto_to_apply = line.price * (line.discount / 100);
                var unit_price = line.price - dto_to_apply;
                var price_obj = this.calculate_unit_price(line, quota_qty, customer_type, true);

                // get vat amount 0.21 or 0.105
                tax_id = line.product.taxes_id[0];
                tax_amount = 1 + (this.pos.taxes_by_id[tax_id].amount);  // 1.21 or 1.105

                // compute quota_amount
                //var unit_price = price_obj.unit_price;
                if (dto_to_apply == 0) {
                    line.price = unit_price;
                }
                var cash_amount = price_obj.cash_amount;
                //
                // var not_credipaz_financial_payments = paymentlines.filter(function(p) {
                //     return p.cashregister.journal.pos_payment_type === 'financial' && p.cashregister.journal.credipaz_card === false;
                // })
                // var not_credipaz_financial_payments_amount = not_credipaz_financial_payments.reduce(function(vant, vact, iact, arr) { return vant + vact['amount'] }, 0)
                // cash_amount += not_credipaz_financial_payments_amount
                var price_with_tax = line.get_price_with_tax();
                var financed_amount = price_with_tax - cash_amount;
                var quota_amount = financed_amount / quota_qty;
                //quota_summary = _t("Financed: ") + quota_qty.toString() + " x " + this.format_currency(quota_amount);
                //console.log(quota_amount, price_obj.unit_price);
                var pu_to_use = 0;
                if (line.quantity == 1) {
                    if (line.product.no_interest) {
                        //pu_to_use = line.price;
                        pu_to_use = line.get_all_prices().priceWithoutTax;
                    } else {
                        pu_to_use = price_obj.unit_price;
                    }
                    line.quota_amount = (parseFloat((pu_to_use * line.quantity * tax_amount).toFixed(2)) - cash_amount) / quota_qty;
                } else {
                    if (line.product.no_interest) {
                        pu_to_use = line.price;
                    } else {
                        pu_to_use = price_obj.unit_price;
                    }
                    if (cash_amount != 0) {
                        var fixed_pu = ((pu_to_use - unit_price) / line.quantity) + unit_price;
                        line.quota_amount = (parseFloat((fixed_pu * line.quantity * tax_amount).toFixed(2)) - cash_amount) / quota_qty;
                    } else {
                        line.quota_amount = (parseFloat((pu_to_use * line.quantity * tax_amount).toFixed(2)) - cash_amount) / quota_qty;
                    }
                }
                //line.set_unit_price(unit_price);
            }
            else {
                var cash_amount_multi = this.get_cash_amount();
                for (var i=0; i < orderlines.length; i++) {
                    var line = orderlines[i];
                // get vat amount 0.21 or 0.105
                tax_id = line.product.taxes_id[0];
                tax_amount = 1 + (this.pos.taxes_by_id[tax_id].amount);
                    // var unit_price = line.price;
                    // var unit_price = line.get_all_prices().priceWithoutTax;
                    var unit_price = line.product.price;
                    // var price_obj = this.calculate_unit_price(line, quota_qty, customer_type, false);
                    var price_obj2 = this.calculate_unit_price(line, quota_qty, customer_type, true);
                    // var quota_amount = price_obj.quota_amount;
                    //console.log(quota_amount, price_obj.unit_price);
                    //line.quota_amount = quota_amount;
                    if (unit_price == 0) {
                        line.quota_amount = 0;
                    } else {
                        if (cash_amount_multi >= (price_obj2.unit_price * line.quantity * tax_amount)) {
                            var cash_amount_to_discount = (price_obj2.unit_price * line.quantity * tax_amount);
                            cash_amount_multi -= cash_amount_to_discount;
                        } else {
                            var cash_amount_to_discount = cash_amount_multi;
                            cash_amount_multi = 0;
                        }
                        line.quota_amount = ((price_obj2.unit_price * line.quantity * tax_amount) - cash_amount_to_discount) / quota_qty;
                        line.price = unit_price;
                    }
                    //line.set_unit_price(unit_price);
                    //line.quota_summary = '';
                }
            }
            var financing_payments = paymentlines.filter(function(p) {
                return p.cashregister.journal.pos_payment_type === 'financial' && p.cashregister.journal.credipaz_card === true;
            })
            if ( financing_payments.length >= 1 ) {
                var last_paymentline = financing_payments[financing_payments.length - 1];
                var due_left = currentOrder.getDueLeft();
                last_paymentline.set_amount(due_left + last_paymentline.amount);
                this.rerender_paymentline(last_paymentline);
            };
            this.update_financing_summary();
        },

        update_financing_summary: function () {
            var currentOrder = this.pos.get('selectedOrder');
            var quotas_amount = currentOrder.get_quotas_total();
            var quotas_summary = currentOrder.quota_qty.toString() + " x " + this.format_currency(quotas_amount);
            this.$('.payment-financing').html(quotas_summary);
            this.$('.payment-financing-total').html(this.format_currency(quotas_amount * currentOrder.quota_qty));
        },

        update_payment_summary: function () {
            var ret = this._super();
            this.update_financing_summary();
            return ret;
        },

        renderElement: function(){
            this._super();

            var self = this;
            var currentOrder = self.pos.get('selectedOrder');
            var quotas_selection = self.pos_widget.payment_screen.$('.quotas-selection');
            // hide quotas that are not available for this order
            var available_quotas = currentOrder.getAvailableQuotas();
            self.pos_widget.payment_screen.$('.quotas-selection option').each(
                    function(){
                        if (!(available_quotas.includes(parseInt(this.value)))){
                            $(this).hide();
                        } else {
                            $(this).show();
                        }
                    });
            // set selection value = quota_qty when rendering the view
            quotas_selection[0].value = currentOrder.quota_qty;
            quotas_selection.change(function() {
                var selected_value = quotas_selection[0].value
                self.change_quota_qty(parseInt(selected_value, 10));
            });
        },
        show: function () {
            var self = this;
            this._super();
            var quotas_selection = self.pos_widget.payment_screen.$('.quotas-selection');
            var currentOrder = self.pos.get('selectedOrder');
            var available_quotas = currentOrder.getAvailableQuotas();
            // hide quotas that are not available for this order
            self.pos_widget.payment_screen.$('.quotas-selection option').each(
                    function(){
                        if (!(available_quotas.includes(parseInt(this.value)))){
                            $(this).hide();
                        } else {
                            $(this).show();
                        }
                    });
            // set selection value = quota_qty when rendering the view
            quotas_selection[0].value = currentOrder.quota_qty;
            quotas_selection.change(function() {
                var selected_value = quotas_selection[0].value
                self.change_quota_qty(parseInt(selected_value, 10));
            });
        }

    });
}
