/******************************************************************************
 *    Financing - Financing on Odoo
 *    Copyright (C) 2017 Eynes
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU Affero General Public License as
 *    published by the Free Software Foundation, either version 3 of the
 *    License, or (at your option) any later version.
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/

function financing_widgets(instance, module) {

    var QWeb = instance.web.qweb,
    _t = instance.web._t;
    var module = instance.point_of_sale;
    var round_di = instance.web.round_decimals;

    module.PaymentScreenWidget.include({
        validate_order: function(options) {
            var self = this;
            options = options || {};

            var currentOrder = this.pos.get('selectedOrder');

            if( !currentOrder.get_client()){
                self.pos_widget.screen_selector.show_popup('error',{
                    message: _t('An anonymous order cannot be confirmed'),
                    comment: _t('Please select a client for this order. This can be done by clicking the order tab')
                });
                return;
            }
            // else
            return this._super(options);
        }
    });

    module.ClientListScreenWidget.include({
        show: function(){
            var self = this;
            currOrder = self.pos.get('selectedOrder');
            if (!currOrder.attributes.client || 'Consumidor Final' === currOrder.attributes.client.name) {
                return self._super();
            }
            self.pos_widget.screen_selector.show_popup('error',{
                message: _t('Cannot change a customer previously defined'),
                comment: _t('Please delete the current order to change the customer')
            });
            return;
        },
    });

    module.OrderButtonWidget = module.OrderButtonWidget.extend({
        selectOrder: function (event) {
            // this function is called when creating or changing between orders 
            this._super(event);
            var partner = this.order.get_client() ? this.order.get_client() : false;
            this.pos.financing_engine.show_product_financing(partner);
        },
        renderElement: function () {
            this._super();
            var order = posmodel.get_order();
            if(order && order.get_client()) {
                order.update_partner_ui_tooltip(order);
            }
        }
    });

    module.PosWidget.include({
        build_widgets: function(){
            this._super();
            var order = this.pos.get('selectedOrder');
            var customer = null;
            if(order) {
                customer = order.get_client();
            }
            this.pos.financing_engine.show_product_financing(customer);
        },
    });

    module.ProductListWidget.include({
        init: function (parent, options) {
            this._super(parent, options);
            this.display_price_with_taxes = false;
            if (
                posmodel
                && posmodel.config
                && posmodel.config.display_price_with_taxes
            ) {
                this.display_price_with_taxes
                    = posmodel.config.display_price_with_taxes
            }
        },
        // renderElement: function () {
        //     this._super();
        //     var order = posmodel.get_order();
        //     var customer = null;
        //     if(order) {
        //         customer = order.get_client();
        //     }
        //     this.pos.pricelist_engine.update_products_ui(customer);
        // }
    });
}
